=== modified file 'bzrlib/tests/test_utextwrap.py'
--- bzrlib/tests/test_utextwrap.py	2011-05-04 15:13:24 +0000
+++ bzrlib/tests/test_utextwrap.py	2011-05-04 17:55:14 +0000
@@ -42,8 +42,8 @@
         self.assertWidth(_str_SD, 13)
 
     def test__break_cjkword(self):
-        self.assertEqual(utextwrap._break_cjkword(u"hello", 3), None)
-        self.assertEqual(utextwrap._break_cjkword(_str_D, 1), None)
+        self.assertEqual(utextwrap._break_cjkword(u"hello", 3), ('', _str_S))
+        self.assertEqual(utextwrap._break_cjkword(_str_D, 1), ('', _str_D))
 
         half = _str_D[:2], _str_D[2:]
         self.assertEqual(utextwrap._break_cjkword(_str_D, 4), half)
@@ -54,6 +54,33 @@
         self.assertEqual(utextwrap._break_cjkword(_str_DS, 10),
                          (_str_D, _str_S))
 
+        self.assertEqual(utextwrap._break_cjkword(_str_D, 8), (_str_D, ''))
+
     def test_wrap(self):
         self.assertEqual(utextwrap.wrap(_str_D, 1), list(_str_D))
         self.assertEqual(utextwrap.wrap(_str_D, 2), list(_str_D))
+        self.assertEqual(utextwrap.wrap(_str_D, 3), list(_str_D))
+        self.assertEqual(utextwrap.wrap(_str_D, 3, break_long_words=False),
+                list(_str_D))
+
+
+# Regression test with Python's test_textwrap
+# Note that some distribution including Ubuntu doesn't install
+# Python's test suite.
+try:
+    import test.test_textwrap as _test_textwrap
+
+    # replace test_textwrap's TextWrapper with UTextWrapper
+    _test_textwrap.TextWrapper = utextwrap.UTextWrapper
+    _test_textwrap.wrap = utextwrap.wrap
+    _test_textwrap.fill = utextwrap.fill
+
+    class TestWrap(_test_textwrap.WrapTestCase):
+        pass
+    class TestLongWord(_test_textwrap.LongWordTestCase):
+        pass
+    class TestIndent(_test_textwrap.IndentTestCases):
+        pass
+except ImportError:
+    pass
+

=== modified file 'bzrlib/utextwrap.py'
--- bzrlib/utextwrap.py	2011-05-04 15:13:24 +0000
+++ bzrlib/utextwrap.py	2011-05-04 17:55:14 +0000
@@ -22,13 +22,35 @@
 __all__ = ["UTextWrapper", "fill", "wrap"]
 
 def _width(s):
+    """Returns width for s.
+    
+    When s is unicode, take care of east asian width.
+    When s is bytes, treat all byte is single width character.
+
+    NOTE: Supporting byte string should be removed with Python 3.
+    """
+    if isinstance(s, str):
+        return len(s)
+    assert isinstance(s, unicode)
     w = 0
     for c in map(_eawidth, s):
-        #w += 2 if _eawidth(c) in 'FWA' else 1 # needs Python >= 2.5
+        #w += 2 if c in 'FWA' else 1 # needs Python >= 2.5
         w += (c in 'FWA' and 2) or 1
     return w
 
 def _break_cjkword(word, width):
+    """Split `word` by `width`.
+
+    Returns a tuple contains 2 strings. First string is head of
+    `word` that's length is less than `width`. Second string is
+    rest of `word`.
+
+    The border of head and rest is next to double width character.
+    Because spaces is not used as word separator on CJK.
+
+    When ``_width(word) < width``, returns ``(word, '')``.
+    When can't split anywhere, returns ``('', word)``.
+    """
     w = 0
     for pos, c in enumerate(word):
         nw = _width(c)
@@ -36,7 +58,7 @@
             break
         w += nw
     else:
-        return word, u''
+        return word, ''
     if pos>0 and _width(word[pos]) == 2:
         # "sssDDD" and pos=3 => "sss", "DDD" (D is double width)
         return word[:pos], word[pos:]
@@ -44,7 +66,7 @@
     while pos > 0 and _width(word[pos-1]) != 2:
         pos -= 1
     if pos == 0:
-        return None
+        return '', word
     return word[:pos], word[pos:]
 
 
@@ -58,11 +80,12 @@
     """
 
     def _handle_long_word(self, chunks, cur_line, cur_len, width):
-        broken_cjk = _break_cjkword(chunks[-1], width)
-        if broken_cjk is not None:
+        head, rest = _break_cjkword(chunks[-1], width)
+        if head:
             chunks.pop()
-            chunks.append(broken_cjk[1])
-            chunks.append(broken_cjk[0])
+            if rest:
+                chunks.append(rest)
+            chunks.append(head)
             return
         textwrap.TextWrapper._handle_long_word(
                 self, chunks, cur_line, cur_len, width)
@@ -110,11 +133,12 @@
                 # Nope, this line is full.
                 else:
                     # break CJK words
-                    cjk_broken = _break_cjkword(chunks[-1], width-cur_len)
-                    if cjk_broken is not None:
-                        cur_line.append(cjk_broken[0])
-                        cur_len += _width(cjk_broken[0])
-                        chunks[-1] = cjk_broken[1]
+                    head, rest = _break_cjkword(chunks[-1], width-cur_len)
+                    if head:
+                        cur_line.append(head)
+                        cur_len += _width(head)
+                        assert rest
+                        chunks[-1] = rest
                     break
 
             # The current line is full, and the next chunk is too big to

